﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using com.bodurov.SortEvolver.Processor;

namespace com.bodurov.SortEvolver.EvolutionEngine
{
    public class Engine
    {
        private readonly List<IntArrayHolder> _statePool;
        private double _initialSortState = Double.MinValue;

        public Engine(IntArraySortTester tester)
        {
            this.Tester = tester;
            this._statePool = new List<IntArrayHolder>();
         }

        public IntArraySortTester Tester { get; private set; }
        public IEnumerable<IntArrayHolder> StatePool { get { return this._statePool; } }
        public int NumberTestStates { get { return 100; } }

        public void GenerateTestData()
        {
            // add one sorted array
            this._statePool.Add(this.Tester.GenerateSorted());
            // add unsorted arrays
            Parallel.For(0, this.NumberTestStates - 1, i => this._statePool.Add(this.Tester.GenerateUnsorted()));
            
        }


        public double InitialSortState
        {
            get
            {
                if(this._initialSortState == Double.MinValue)
                {
                    long sumOfAllLong = 0;
                    const int SHIFT = 1000000; // we need that because Interlocked.Add has no overload for double
                    Parallel.ForEach<IntArrayHolder, double>(
                        this._statePool,
                        () => 0,
                        (current, index, pState) => this.Tester.Evaluate(current),
                        finalResult => Interlocked.Add(ref sumOfAllLong, (long)(finalResult * SHIFT)));

                    // unshift
                    sumOfAllLong = sumOfAllLong / SHIFT;
                    this._initialSortState = sumOfAllLong / (double)this._statePool.Count;
                }
                return this._initialSortState;
            }
        }

        public double Test(IIntArraySorter sorter)
        {
            long sumOfAllLong = 0;
            const int SHIFT = 1000000; // we need that because Interlocked.Add has no overload for double
            Parallel.ForEach<IntArrayHolder, double>(
                this._statePool, 
                () => 0, 
                (current, index, pState) => this.Tester.Evaluate(sorter.Run(current)),
                finalResult => Interlocked.Add(ref sumOfAllLong, (long)(finalResult * SHIFT)));

            // unshift
            sumOfAllLong = sumOfAllLong/SHIFT;
            return sumOfAllLong/(double)this._statePool.Count;
        }


    }
}
